import { JsonRpc, RpcError } from 'eosjs';
import { PublicKey } from 'eosjs/dist/eosjs-key-conversions';
import { KeyType } from 'eosjs/dist/eosjs-numeric';
import base64url from 'base64url';

function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];
      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }
    return target;
  };
  return _extends.apply(this, arguments);
}
function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}
function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;
  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];
  return arr2;
}
function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];
  if (it) return (it = it.call(o)).next.bind(it);
  if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
    if (it) o = it;
    var i = 0;
    return function () {
      if (i >= o.length) return {
        done: true
      };
      return {
        done: false,
        value: o[i++]
      };
    };
  }
  throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

// "integers are represented using the base64url encoding of their big-endian representations"
// https://datatracker.ietf.org/doc/html/rfc7517#appendix-A.1
function bnToBase64Url(bn) {
  var buffer = bn.toArrayLike(Buffer, 'be');
  return base64url(buffer);
}

var eos = {
	chainId: "aca376f206b8fc25a6ed44dbdc66547c36c6c33e3a119ffbeaef943642f0e906",
	service: [
		{
			id: "https://eos.greymass.com",
			type: "LinkedDomains",
			serviceEndpoint: "https://eos.greymass.com"
		},
		{
			id: "https://eos.dfuse.eosnation.io",
			type: "LinkedDomains",
			serviceEndpoint: "https://eos.dfuse.eosnation.io"
		}
	]
};
var telos = {
	chainId: "4667b205c6838ef70ff7988f6e8257e8be0e1284a2f59699054a018f743b1d11",
	service: [
		{
			id: "https://telos.greymass.com",
			type: "LinkedDomains",
			serviceEndpoint: "https://telos.greymass.com"
		}
	]
};
var europechain = {
	chainId: "f778f7d2f124b110e0a71245b310c1d0ac1a0edd21f131c5ecb2e2bc03e8fe2e",
	service: [
		{
			id: "https://api.xec.cryptolions.io",
			type: "LinkedDomains",
			serviceEndpoint: "https://api.xec.cryptolions.io"
		}
	]
};
var antelopeChainRegistry = {
	eos: eos,
	"eos:testnet:jungle": {
	chainId: "2a02a0053e5a8cf73a56ba0fda11e4d92e0238a4a2aa74fccf46d5a910746840",
	service: [
		{
			id: "https://jungle4.cryptolions.io",
			type: "LinkedDomains",
			serviceEndpoint: "https://jungle4.cryptolions.io"
		}
	]
},
	telos: telos,
	europechain: europechain
};

const _iteratorSymbol = typeof Symbol !== "undefined" ? Symbol.iterator || (Symbol.iterator = Symbol("Symbol.iterator")) : "@@iterator";
function _settle(pact, state, value) {
  if (!pact.s) {
    if (value instanceof _Pact) {
      if (value.s) {
        if (state & 1) {
          state = value.s;
        }
        value = value.v;
      } else {
        value.o = _settle.bind(null, pact, state);
        return;
      }
    }
    if (value && value.then) {
      value.then(_settle.bind(null, pact, state), _settle.bind(null, pact, 2));
      return;
    }
    pact.s = state;
    pact.v = value;
    var observer = pact.o;
    if (observer) {
      observer(pact);
    }
  }
}
var _Pact = /*#__PURE__*/function () {
  function _Pact() {}
  _Pact.prototype.then = function (onFulfilled, onRejected) {
    var result = new _Pact();
    var state = this.s;
    if (state) {
      var callback = state & 1 ? onFulfilled : onRejected;
      if (callback) {
        try {
          _settle(result, 1, callback(this.v));
        } catch (e) {
          _settle(result, 2, e);
        }
        return result;
      } else {
        return this;
      }
    }
    this.o = function (_this) {
      try {
        var value = _this.v;
        if (_this.s & 1) {
          _settle(result, 1, onFulfilled ? onFulfilled(value) : value);
        } else if (onRejected) {
          _settle(result, 1, onRejected(value));
        } else {
          _settle(result, 2, value);
        }
      } catch (e) {
        _settle(result, 2, e);
      }
    };
    return result;
  };
  return _Pact;
}();
function _isSettledPact(thenable) {
  return thenable instanceof _Pact && thenable.s & 1;
}
function _forTo(array, body, check) {
  var i = -1,
    pact,
    reject;
  function _cycle(result) {
    try {
      while (++i < array.length && (!check || !check())) {
        result = body(i);
        if (result && result.then) {
          if (_isSettledPact(result)) {
            result = result.v;
          } else {
            result.then(_cycle, reject || (reject = _settle.bind(null, pact = new _Pact(), 2)));
            return;
          }
        }
      }
      if (pact) {
        _settle(pact, 1, result);
      } else {
        pact = result;
      }
    } catch (e) {
      _settle(pact || (pact = new _Pact()), 2, e);
    }
  }
  _cycle();
  return pact;
}
function _forOf(target, body, check) {
  if (typeof target[_iteratorSymbol] === "function") {
    var _cycle = function _cycle(result) {
      try {
        while (!(step = iterator.next()).done && (!check || !check())) {
          result = body(step.value);
          if (result && result.then) {
            if (_isSettledPact(result)) {
              result = result.v;
            } else {
              result.then(_cycle, reject || (reject = _settle.bind(null, pact = new _Pact(), 2)));
              return;
            }
          }
        }
        if (pact) {
          _settle(pact, 1, result);
        } else {
          pact = result;
        }
      } catch (e) {
        _settle(pact || (pact = new _Pact()), 2, e);
      }
    };
    var iterator = target[_iteratorSymbol](),
      step,
      pact,
      reject;
    _cycle();
    if (iterator["return"]) {
      var _fixup = function _fixup(value) {
        try {
          if (!step.done) {
            iterator["return"]();
          }
        } catch (e) {}
        return value;
      };
      if (pact && pact.then) {
        return pact.then(_fixup, function (e) {
          throw _fixup(e);
        });
      }
      _fixup();
    }
    return pact;
  }
  // No support for Symbol.iterator
  if (!("length" in target)) {
    throw new TypeError("Object is not iterable");
  }
  // Handle live collections properly
  var values = [];
  for (var i = 0; i < target.length; i++) {
    values.push(target[i]);
  }
  return _forTo(values, function (i) {
    return body(values[i]);
  }, check);
}
function _catch(body, recover) {
  try {
    var result = body();
  } catch (e) {
    return recover(e);
  }
  if (result && result.then) {
    return result.then(void 0, recover);
  }
  return result;
}
var resolve = function resolve(did, parsed,
// @ts-ignore(TS6133 declared but never used)
resolver, options) {
  try {
    var registry = _extends({}, antelopeChainRegistry, options.antelopeChainRegistry);
    var methodId = checkDID(parsed, registry);
    if (options.antelopeChainUrl) {
      var _parsed$id$split = parsed.id.split(':'),
        chainId = _parsed$id$split[0],
        subject = _parsed$id$split[1];
      methodId = {
        chain: {
          chainId: chainId,
          service: []
        },
        subject: subject
      };
    }
    if (!methodId) {
      // invalid method-specific-id OR no matching chain in the registry
      return Promise.resolve(getResolutionError('invalidDid'));
    }
    return Promise.resolve(fetchAccount(methodId, did, parsed, options)).then(function (antelopeAccount) {
      if (!antelopeAccount) {
        return getResolutionError('notFound');
      }
      var didDoc = createDIDDocument(methodId, parsed.did, antelopeAccount);
      return {
        didResolutionMetadata: {
          contentType: 'application/did+ld+json'
        },
        didDocument: didDoc,
        didDocumentMetadata: {}
      };
    });
  } catch (e) {
    return Promise.reject(e);
  }
};
var createRpcFetchAccount = function createRpcFetchAccount(methodId, service, options) {
  try {
    var rpcOptions = {};
    if (options.fetch) rpcOptions.fetch = options.fetch;
    var endpoint = service.serviceEndpoint;
    var rpc = new JsonRpc(endpoint, rpcOptions);
    return Promise.resolve(_catch(function () {
      return Promise.resolve(rpc.get_account(methodId.subject));
    }, function (e) {
      if (e instanceof RpcError) {
        var _error$error, _error$error2, _error$error2$details, _error$error2$details2;
        var error = e.json;
        if (error.code === 500 && ((_error$error = error.error) == null ? void 0 : _error$error.code) === 0 && (_error$error2 = error.error) != null && (_error$error2$details = _error$error2.details[0]) != null && (_error$error2$details2 = _error$error2$details.message) != null && _error$error2$details2.startsWith('unknown key')) {
          // Antelope v3+ returns 500 error when account does not exist
          // Eosio v2 (with eosjs) returns null when account does not exist
          // TODO update to latest eosjs to fix this...???
          return null;
        }
      }
      console.error(JSON.stringify(e, null, 2));
      // then try other services in case of error.
    }));
  } catch (e) {
    return Promise.reject(e);
  }
};
var fetchAccount = function fetchAccount(methodId, _did, _parsed, options) {
  try {
    var _temp3 = function _temp3(_result) {
      var _exit2;
      if (_exit) return _result;
      var services = findServices(methodId.chain.service, serviceType);
      var _temp = _forOf(services, function (service) {
        return Promise.resolve(createRpcFetchAccount(methodId, service, options)).then(function (_await$createRpcFetch2) {
          _exit2 = 1;
          return _await$createRpcFetch2;
        });
      }, function () {
        return _exit2;
      });
      return _temp && _temp.then ? _temp.then(function (_result2) {
        return _exit2 ? _result2 : null;
      }) : _exit2 ? _temp : null;
    };
    var _exit;
    var serviceType = 'LinkedDomains';
    var _temp2 = function () {
      if (options.antelopeChainUrl) {
        return Promise.resolve(createRpcFetchAccount(methodId, {
          serviceEndpoint: options.antelopeChainUrl
        }, options)).then(function (_await$createRpcFetch) {
          _exit = 1;
          return _await$createRpcFetch;
        });
      }
    }();
    return Promise.resolve(_temp2 && _temp2.then ? _temp2.then(_temp3) : _temp3(_temp2));
  } catch (e) {
    return Promise.reject(e);
  }
};
var PATTERN_ACCOUNT_NAME = "([a-z1-5.]{0,12}[a-z1-5])";
var PATTERN_CHAIN_ID = "([A-Fa-f0-9]{64})";
var PATTERN_CHAIN_NAME = "(([a-z1-5.]{0,12}[a-z1-5])((:[a-z1-5.]{0,12}[a-z1-5])+)?)";
function toRegExp(pattern) {
  return new RegExp("^" + pattern + "$");
}
var REGEX_ACCOUNT_NAME = toRegExp(PATTERN_ACCOUNT_NAME);
var REGEX_CHAIN_ID = toRegExp(PATTERN_CHAIN_ID);
var REGEX_CHAIN_NAME = toRegExp(PATTERN_CHAIN_NAME);
var REGEX_ID_AND_SUBJECT = toRegExp(PATTERN_CHAIN_ID + ":" + PATTERN_ACCOUNT_NAME);
var REGEX_NAME_AND_SUBJECT = toRegExp(PATTERN_CHAIN_NAME + ":" + PATTERN_ACCOUNT_NAME);
var CONDITIONAL_PROOF_2022 = 'ConditionalProof2022';
function getResolutionError(error) {
  return {
    didResolutionMetadata: {
      error: error
    },
    didDocument: null,
    didDocumentMetadata: {}
  };
}
function checkDID(parsed, registry) {
  // findChainByName
  var partsName = parsed.id.match(REGEX_NAME_AND_SUBJECT);
  if (partsName) {
    var entry = registry[partsName[1]];
    if (entry) return {
      chain: entry,
      subject: partsName[partsName.length - 1]
    };
    return undefined;
  }
  // findChainById
  var partsID = parsed.id.match(REGEX_ID_AND_SUBJECT);
  if (partsID) {
    for (var _i = 0, _Object$keys = Object.keys(registry); _i < _Object$keys.length; _i++) {
      var key = _Object$keys[_i];
      var _entry = registry[key];
      if (_entry.chainId === partsID[1]) return {
        chain: _entry,
        subject: partsID[partsID.length - 1]
      };
    }
  }
  return undefined;
}
function findServices(service, type) {
  return service.filter(function (s) {
    return Array.isArray(s.type) ? s.type.includes(type) : s.type === type;
  });
}
function getCurveNamesFromType(type) {
  switch (type) {
    case KeyType.k1:
      return {
        jwkCurve: 'secp256k1',
        verificationMethodType: 'EcdsaSecp256k1VerificationKey2019'
      };
    case KeyType.r1:
      return {
        jwkCurve: 'P-256',
        verificationMethodType: 'JsonWebKey2020'
      };
    case KeyType.wa:
      return {
        jwkCurve: 'P-256',
        verificationMethodType: 'JsonWebKey2020'
      };
  }
}
function createKeyMethod(baseId, i, did, key) {
  var pubKey = PublicKey.fromString(key);
  var ecPubKey = pubKey.toElliptic();
  if (!pubKey.isValid()) throw new Error('Key is not valid');
  var _getCurveNamesFromTyp = getCurveNamesFromType(pubKey.getType()),
    jwkCurve = _getCurveNamesFromTyp.jwkCurve,
    verificationMethodType = _getCurveNamesFromTyp.verificationMethodType;
  var publicKeyJwk = {
    crv: jwkCurve,
    kty: 'EC',
    x: bnToBase64Url(ecPubKey.getPublic().getX()),
    y: bnToBase64Url(ecPubKey.getPublic().getY()),
    kid: pubKey.toString()
  };
  var keyMethod = {
    id: baseId + '-' + i,
    controller: did,
    type: verificationMethodType,
    publicKeyJwk: publicKeyJwk
  };
  return keyMethod;
}
function createAccountMethod(baseId, methodId, i, did, account) {
  var delegatedChain = baseId.slice(0, baseId.lastIndexOf(methodId.subject) - 1);
  var accountMethod = {
    id: baseId + '-' + i,
    controller: did,
    type: CONDITIONAL_PROOF_2022,
    conditionDelegated: delegatedChain + ':' + account.permission.actor + '#' + account.permission.permission
  };
  return accountMethod;
}
function createDIDDocument(methodId, did, antelopeAccount) {
  var verificationMethod = [];
  for (var _iterator = _createForOfIteratorHelperLoose(antelopeAccount.permissions), _step; !(_step = _iterator()).done;) {
    var permission = _step.value;
    var baseId = did + '#' + permission.perm_name;
    permission.required_auth.accounts = permission.required_auth.accounts.filter(function (account) {
      return account.permission.permission !== 'eosio.code';
    });
    var method = void 0;
    if (permission.required_auth.keys.length === 1 && permission.required_auth.accounts.length === 0) {
      method = createKeyMethod(baseId, 0, did, permission.required_auth.keys[0].key);
      method.id = baseId;
    } else {
      method = {
        id: baseId,
        controller: did,
        type: CONDITIONAL_PROOF_2022,
        threshold: permission.required_auth.threshold,
        conditionWeightedThreshold: []
      };
      if (permission.parent !== '') {
        method.relationshipParent = [did + '#' + permission.parent];
      }
      var i = 0;
      for (var _iterator2 = _createForOfIteratorHelperLoose(permission.required_auth.keys), _step2; !(_step2 = _iterator2()).done;) {
        var key = _step2.value;
        method.conditionWeightedThreshold.push({
          condition: createKeyMethod(baseId, i, did, key.key),
          weight: key.weight
        });
        i++;
      }
      for (var _iterator3 = _createForOfIteratorHelperLoose(permission.required_auth.accounts), _step3; !(_step3 = _iterator3()).done;) {
        var account = _step3.value;
        method.conditionWeightedThreshold.push({
          condition: createAccountMethod(baseId, methodId, i, did, account),
          weight: account.weight
        });
        i++;
      }
    }
    verificationMethod.push(method);
  }
  var doc = {
    '@context': ['https://www.w3.org/ns/did/v1', 'https://w3c-ccg.github.io/verifiable-conditions/contexts/verifiable-conditions-2021-v1.json'],
    id: did,
    verificationMethod: verificationMethod,
    service: methodId.chain.service
  };
  return doc;
}
function createResolver(options) {
  if (options === void 0) {
    options = {};
  }
  return function (did, parsed,
  // @ts-ignore(TS6133 declared but never used)
  resolver, antelopeOptions) {
    return resolve(did, parsed, resolver, _extends({}, antelopeOptions, options));
  };
}

function getResolver(options) {
  return {
    eosio: resolve,
    antelope: createResolver(options)
  };
}

export { CONDITIONAL_PROOF_2022, REGEX_ACCOUNT_NAME, REGEX_CHAIN_ID, REGEX_CHAIN_NAME, antelopeChainRegistry, checkDID, createDIDDocument, fetchAccount, getResolver };
//# sourceMappingURL=index.module.js.map
